require 'test_helper'

class CategoryControllerTest < ActionDispatch::IntegrationTest
  test "GET fake category returns 404" do
    get category_path(blog_name: blogs(:one).urlName,
                      cat_name: "fake")
    assert_response :not_found
  end

  test "GET category with fake blog name returns 404" do
    get category_path(blog_name: "fake",
                      cat_name: categories(:one).path)
    assert_response :not_found
  end

  test "GET category with wrong blog name returns 404" do
    get category_path(blog_name: blogs(:two).urlName,
                      cat_name: categories(:one).path)
    assert_response :not_found
  end

  test "GET category with no posts returns empty" do
    get category_path(blog_name: blogs(:one).urlName,
                      cat_name: categories(:two).path)
    assert_response :success
    assert_equal 0, assigns(:posts).count
  end

  test "GET category with multiple posts returns them" do
    c = categories(:one)
    c.posts << posts(:one)
    c.posts << posts(:three)
    get category_path(c.to_param)
    assert_response :success
    assert_equal [posts(:three), posts(:one)], c.posts
  end

  test "GET category doesn't return draft posts" do
    c = categories(:one)
    c.posts << posts(:draft)
    get category_path(c.to_param)
    assert_response :success
    assert_equal 0, assigns(:posts).count
  end

  test "MANAGE redirects to login when not logged in" do
    get manage_category_path(blog_name: blogs(:one).urlName)
    assert_redirected_to login_path
    assert_equal "Please login first.", flash[:alert]
  end

  test "MANAGE returns 403 when logged in as other user" do
    post login_path(login: {
                      username: "user_one",
                      password: "password1"
                    })
    get manage_category_path(blog_name: blogs(:two).urlName)
    assert_response :forbidden
    assert_equal "You do not have access to that page!", flash[:alert]
  end

  test "MANAGE returns 403 when blog doesn't exist" do
    # If we returned 404, it would be possible to use a brute force attack
    # to find blogs belonging to other users
    post login_path(login: {
                      username: "user_one",
                      password: "password1"
                    })
    get manage_category_path(blog_name: "definitely_dne")
    assert_response :forbidden
    assert_equal "You do not have access to that page!", flash[:alert]
  end

  test "MANAGE returns 200 when user is logged in to correct account" do
    post login_path(login: {
                      username: "user_one",
                      password: "password1"
                    })
    get manage_category_path(blog_name: blogs(:one).urlName)
    assert_response :success
  end

  test "CREATE redirects to login when not logged in" do
    post manage_category_path(blog_name: blogs(:one).urlName,
                       category: {name: "test category"})
    assert_redirected_to login_path
    assert_nil Category.find_by_name "test category"
  end

  test "CREATE returns 403 when logged in as other user" do
    post login_path(login: {
                      username: "user_one",
                      password: "password1"
                    })
    post manage_category_path(blog_name: blogs(:two).urlName,
                              category: {name: "test category"})
    assert_response :forbidden
    assert_nil Category.find_by_name "test category"
  end

  test "CREATE returns 403 when blog doesn't exist" do
    post login_path(login: {
                      username: "user_one",
                      password: "password1"
                    })
    post manage_category_path(blog_name: "definitely_dne",
                              category: {name: "test category"})
    assert_response :forbidden
    assert_nil Category.find_by_name "test category"
  end

  test "CREATE flashes alerts on error" do
    post login_path(login: {
                      username: "user_one",
                      password: "password1"
                    })
    # Name too short
    post manage_category_path(blog_name: blogs(:one).urlName,
                              category: {name: "hi"})
    assert_response :success
    assert_equal ["Name is too short (minimum is 3 characters)"],
                 flash[:alerts]
    assert_nil Category.find_by_name "hi"
  end

  test "CREATE returns 200 when user is logged in correctly" do
    post login_path(login: {
                      username: "user_one",
                      password: "password1"
                    })
    post manage_category_path(blog_name: blogs(:one).urlName,
                              category: {name: "test category"})
    assert_response :success
    assert_not_nil blogs(:one).categories.find_by_name "test category"
  end

  test "DELETE redirects to login when not logged in" do
    delete delete_category_path(blog_name: blogs(:one).urlName,
                                cat_name: categories(:one).path)
    assert_redirected_to login_path
    assert_not_nil Category.find(categories(:one).id)
  end

  test "DELETE returns 403 when logged in as other user" do
    post login_path(login: {
                      username: "user_two",
                      password: "password2"
                    })
    delete delete_category_path(blog_name: blogs(:one).urlName,
                                cat_name: categories(:one).path)
    assert_response :forbidden
    assert_not_nil Category.find(categories(:one).id)
  end

  test "DELETE returns 403 when blog doesn't exist" do
    post login_path(login: {
                      username: "user_one",
                      password: "password1"
                    })
    delete delete_category_path(blog_name: "dne",
                                cat_name: categories(:one).path)
    assert_response :forbidden
    assert_not_nil Category.find_by(id: categories(:one).id)
  end

  test "DELETE redirects to manage when category doesn't exist" do
    post login_path(login: {
                      username: "user_one",
                      password: "password1"
                    })
    delete delete_category_path(blog_name: blogs(:one).urlName,
                                cat_name: "dne")
    assert_redirected_to manage_category_path
    assert_equal "Could not find that category!", flash[:alert]
  end

  test "DELETE returns 200 otherwise" do
    post login_path(login: {
                      username: "user_one",
                      password: "password1"
                    })
    delete delete_category_path(blog_name: blogs(:one).urlName,
                                cat_name: categories(:one).path)
    assert_redirected_to manage_category_path
    assert_nil Category.find_by(id: categories(:one).id)
  end

  test "UPDATE redirects to login when not logged in" do
    og_name = categories(:one).name
    patch update_category_path(blog_name: blogs(:one).urlName,
                               cat_name: categories(:one).path,
                               category: {name: "new name"})
    assert_redirected_to login_path
    assert_equal og_name, Category.find(categories(:one).id).name
  end

  test "UPDATE returns 403 when logged in as other user" do
    og_name = categories(:one).name
    post login_path(login: {
                      username: "user_two",
                      password: "password2"
                    })
    patch update_category_path(blog_name: blogs(:one).urlName,
                               cat_name: categories(:one).path,
                               category: {name: "new name"})
    assert_response :forbidden
    assert_equal og_name, Category.find(categories(:one).id).name
  end

  test "UPDATE returns 403 when blog doesn't exist" do
    og_name = categories(:one).name
    post login_path(login: {
                      username: "user_one",
                      password: "password1"
                    })
    patch update_category_path(blog_name: "dne",
                               cat_name: categories(:one).path,
                               category: {name: "new name"})
    assert_response :forbidden
    assert_equal og_name, Category.find(categories(:one).id).name
  end

  test "UPDATE redirects to manage when category doesn't exist" do
    post login_path(login: {
                      username: "user_one",
                      password: "password1"
                    })
    patch update_category_path(blog_name: blogs(:one).urlName,
                               cat_name: "dne",
                               category: {name: "new name"})
    assert_redirected_to manage_category_path
    assert_equal "Could not find that category!", flash[:alert]
  end

  test "UPDATE returns 200 otherwise" do
    post login_path(login: {
                      username: "user_one",
                      password: "password1"
                    })
    patch update_category_path(blog_name: blogs(:one).urlName,
                               cat_name: categories(:one).path,
                               category: {name: "new name"})
    assert_redirected_to manage_category_path
    assert_equal "new name", Category.find(categories(:one).id).name
  end
end
