require 'test_helper'

class PageControllerTest < ActionDispatch::IntegrationTest
  test "GET return 404 when page doesn't exist" do
    get page_path(blog_name: "blog_two",
                  id: 12345,
                  path: pages(:draft).path)
    assert_response :not_found
  end

  test "GET return 404 when page does not match blog" do
    get page_path(blog_name: "blog_two",
                  id: pages(:one).id,
                  path: pages(:draft).path)
    assert_response :not_found
  end

  test "GET return 404 when page is draft and user is logged out" do
    get page_path(blog_name: "blog_two",
                  id: pages(:draft).id,
                  path: pages(:draft).path)
    assert_response :not_found
  end

  test "GET don't leak title if post is draft and user is logged out" do
    get page_path(blog_name: "blog_two",
                  id: pages(:draft).id,
                  path: pages(:draft).path)
    assert_select "title", pages(:draft).blog.title
  end

  test "GET return 404 when page is draft and user is logged in" do
    post login_path(login: {username: "user_one", password: "password1"})
    get page_path(blog_name: "blog_two",
                  id: pages(:draft).id,
                  path: pages(:draft).path)
    assert_response :not_found
  end

  test "GET don't leak title if post is draft and user is logged in" do
    post login_path(login: {username: "user_one", password: "password1"})
    get page_path(blog_name: "blog_two",
                  id: pages(:draft).id,
                  path: pages(:draft).path)
    assert_select "title", pages(:draft).blog.title
  end

  test "GET return 200 when page is draft and user is logged in as author" do
    post login_path(login: {username: "user_two", password: "password2"})
    get page_path(blog_name: "blog_two",
                  id: pages(:draft).id,
                  path: pages(:draft).path)
    assert_response :success
  end

  test "GET draft page has draft in title" do
    post login_path(login: {username: "user_two", password: "password2"})
    get page_path(blog_name: "blog_two",
                  id: pages(:draft).id,
                  path: pages(:draft).path)
    assert_select "title", "[Draft] My drafted page | Another blog"
  end

  test "GET return 200 when page is published and user is not logged in" do
    get page_path(blog_name: "blog_two",
                  id: pages(:two).id,
                  path: pages(:two).path)
    assert_response :success
  end

  test "GET published page doesn't have draft in title" do
    get page_path(blog_name: "blog_two",
                  id: pages(:two).id,
                  path: pages(:two).path)
    assert_select "title", "The second page | Another blog"
  end

  test "MANAGE return 403 when user is not logged in" do
    get manage_page_path(blog_name: "blog_two")
    assert_response :forbidden
  end

  test "MANAGE return 403 when user is logged in" do
    post login_path(login: {username: "user_one", password: "password1"})
    get manage_page_path(blog_name: "blog_two")
    assert_response :forbidden
  end

  test "MANAGE return 200 when user is logged in as blog author" do
    post login_path(login: {username: "user_two", password: "password2"})
    get manage_page_path(blog_name: "blog_two")
    assert_response :success
  end

  test "NEW return 403 when user is logged out" do
    get new_page_path(blog_name: "blog_two")
    assert_response :forbidden
  end

  test "NEW return 403 when user is logged in as another user" do
    post login_path(login: {username: "user_one", password: "password1"})
    get new_page_path(blog_name: "blog_two")
    assert_response :forbidden
  end

  test "NEW return 200 when user is logged in as author" do
    post login_path(login: {username: "user_two", password: "password2"})
    get new_page_path(blog_name: "blog_two")
    assert_response :success
  end

  test "CREATE don't allow user to create page if logged out" do
    lpage = Page.last.id
    post new_page_path(blog_name: "blog_two",
                       page: {title: "My new page",
                              body: "My new page body"})
    assert_response :forbidden
    assert_equal lpage, Page.last.id
  end

  test "CREATE don't allow user to create page if logged in" do
    lpage = Page.last.id
    post login_path(login: {username: "user_one", password: "password1"})
    post new_page_path(blog_name: "blog_two",
                       page: {title: "My new page",
                              body: "My new page body"})
    assert_response :forbidden
    assert_equal lpage, Page.last.id
  end

  test "CREATE allow user to create new page if logged in as author" do
    lpage = Page.last.id
    post login_path(login: {username: "user_two", password: "password2"})
    post new_page_path(blog_name: "blog_two",
                       page: {title: "My new page",
                              body: "My new page body"})
    assert_redirected_to edit_page_path(id: Page.last.id,
                                        path: Page.last.path)
    assert_not_equal lpage, Page.last.id
    assert_equal "My new page", Page.last.title
    assert_equal "My new page body", Page.last.body
  end

  test "EDIT return 403 if user is logged out" do
    get edit_page_path(blog_name: "blog_two",
                       id: pages(:two).id,
                       path: pages(:two).path)
    assert_response :forbidden
  end

  test "EDIT return 403 if user is logged in" do
    post login_path(login: {username: "user_one", password: "password1"})
    get edit_page_path(blog_name: "blog_two",
                       id: pages(:two).id,
                       path: pages(:two).path)
    assert_response :forbidden
  end

  test "EDIT return 200 if user is logged in as author" do
    post login_path(login: {username: "user_two", password: "password2"})
    get edit_page_path(blog_name: "blog_two",
                       id: pages(:two).id,
                       path: pages(:two).path)
    assert_response :success
  end

  test "UPDATE don't allow if logged out" do
    patch edit_page_path(blog_name: "blog_two",
                        id: pages(:two).id,
                        path: pages(:two).path,
                        page: {title: "New title"})
    assert_response :forbidden
    assert_equal pages(:two).title, Page.find(pages(:two).id).title
    assert_equal pages(:two).body, Page.find(pages(:two).id).body
  end

  test "UPDATE don't allow if logged in" do
    post login_path(login: {username: "user_one", password: "password1"})
    patch edit_page_path(blog_name: "blog_two",
                         id: pages(:two).id,
                         path: pages(:two).path,
                         page: {title: "New title"})
    assert_response :forbidden
    assert_equal pages(:two).title, Page.find(pages(:two).id).title
    assert_equal pages(:two).body, Page.find(pages(:two).id).body
  end

  test "UPDATE allow if logged in as author" do
    post login_path(login: {username: "user_two", password: "password2"})
    patch edit_page_path(blog_name: "blog_two",
                        id: pages(:two).id,
                        path: pages(:two).path,
                        page: {title: "New title"})
    assert_response :success
    assert_equal "New title", Page.find(pages(:two).id).title
    assert_equal pages(:two).body, Page.find(pages(:two).id).body
  end

  test "PRIORITY don't allow if not logged in" do
    post page_priority_path(blog_name: "blog_two",
                            p: 1,
                            dir: :up)
    assert_response :forbidden
    blog = Blog.find(blogs(:two).id)
    assert_equal pages(:two), blog.pages.all[0]
  end

  test "PRIORITY don't allow if user logged in" do
    post login_path(login: {username: "user_one", password: "password1"})
    post page_priority_path(blog_name: "blog_two",
                            p: 3,
                            dir: :up)
    assert_response :forbidden
    blog = Blog.find(blogs(:two).id)
    assert_equal pages(:two), blog.pages.all[0]
    assert_equal pages(:draft), blog.pages.all[1]
  end

  test "PRIORITY allow up if user logged in as author" do
    post login_path(login: {username: "user_two", password: "password2"})
    post page_priority_path(blog_name: "blog_two",
                            p: 3,
                            dir: :up)
    assert_redirected_to manage_page_path("blog_two")
    blog = Blog.find(blogs(:two).id)
    bpages = blog.pages.order(:priority)
    assert_equal pages(:two).id, bpages[1].id
    assert_equal pages(:draft).id, bpages[0].id
  end

  test "PRIORITY allow down if user logged in as author" do
    post login_path(login: {username: "user_two", password: "password2"})
    post page_priority_path(blog_name: "blog_two",
                            p: 2,
                            dir: :down)
    assert_redirected_to manage_page_path("blog_two")
    blog = Blog.find(blogs(:two).id)
    bpages = blog.pages.order(:priority)
    assert_equal pages(:two).id, bpages[1].id
    assert_equal pages(:draft).id, bpages[0].id
  end

  test "DELETE don't allow if not logged in" do
    delete delete_page_path(blog_name: "blog_two",
                            id: pages(:two).id,
                            path: "two_path")
    assert_response :forbidden
    assert_not_nil Page.find_by(id: pages(:two).id)
    assert_equal pages(:two), Page.find_by(id: pages(:two).id)
  end

  test "DELETE don't allow if logged in" do
    post login_path(login: {username: "user_one", password: "password1"})
    delete delete_page_path(blog_name: "blog_two",
                            id: pages(:two).id,
                            path: "two_path")
    assert_response :forbidden
    assert_not_nil Page.find_by(id: pages(:two).id)
    assert_equal pages(:two), Page.find_by(id: pages(:two).id)
  end

  test "DELETE allow if logged in as author" do
    post login_path(login: {username: "user_two", password: "password2"})
    delete delete_page_path(blog_name: "blog_two",
                            id: pages(:two).id,
                            path: "two_path")
    assert_redirected_to manage_page_path("blog_two")
    assert_nil Page.find_by(id: pages(:two).id)
  end

  test "Can create page with no title" do
    post login_path(login: {username: "user_one", password: "password1"})
    post new_page_path(blog_name: "blog_one",
                       page: {body: "This post has no title"})
    new_page = Page.last
    assert_equal "", new_page.title
    get page_path(blog_name: "blog_one",
                  id: new_page.id,
                  path: new_page.path)
    assert_response :success
    assert_select "title", "| " + new_page.blog.title
  end
end
