require 'test_helper'

class PostControllerTest < ActionDispatch::IntegrationTest
  test "return 200 from post index" do
    get blog_path(blog_name: blogs(:one).urlName)
    assert_response :success
  end

  test "return 200 from root when main_blog is set" do
    Rails.configuration.pressr.main_blog_id = blogs(:one).id
    Rails.application.reload_routes!
    get root_path
    assert_response :success
  end

  test "root title is correct when main_blog is set" do
    Rails.configuration.pressr.main_blog_id = blogs(:one).id
    Rails.application.reload_routes!
    get root_path
    assert_select "title", blogs(:one).title
  end

  test "rss feed is correct on post index" do
    get blog_path(blog_name: blogs(:one).urlName)
    assert_select "link", rel: "alternate", title: "RSS",
                  href: blog_path(blog_name: blogs(:one).urlName, format: :rss)
  end

  test "rss feed is correct on root when main_blog is set" do
    Rails.configuration.pressr.main_blog_id = blogs(:one).id
    Rails.application.reload_routes!
    get root_path
    assert_select "link", rel: "alternate", title: "RSS",
                  href: blog_path(blog_name: blogs(:one).urlName, format: :rss)
  end

  test "rss feed returns 200" do
    get blog_path(blog_name: blogs(:one).urlName, format: :rss)
    assert_response :success
  end

  test "rss feed blog attributes are correct" do
    get blog_path(blog_name: blogs(:one).urlName, format: :rss)
    assert_select "rss channel title", blogs(:one).title
    assert_select "rss channel description", blogs(:one).slogan
    assert_select "rss channel link", blog_url(blog_name: blogs(:one).urlName)
  end
  
  test "return 404 if post does not exist" do
    get post_path(post_id: 12345,
                  blog_name: blogs(:one).urlName)
    assert_response :not_found
  end

  test "return 404 if post is draft and user is logged out" do
    get post_path(post_id: posts(:draft).id,
                  blog_name: posts(:draft).blog.urlName)
    assert_response :not_found
  end

  test "don't leak title if post is draft and user is logged out" do
    get post_path(post_id: posts(:draft).id,
                  blog_name: posts(:draft).blog.urlName)
    assert_select "title", posts(:draft).blog.title
  end

  test "return 404 if post is draft and user is logged in" do
    post login_path(login: {username: "user_two", password: "password2"})
    get post_path(post_id: posts(:draft).id,
                  blog_name: posts(:draft).blog.urlName)
    assert_response :not_found
  end

  test "don't leak title if post is draft and user is logged in" do
    post login_path(login: {username: "user_two", password: "password2"})
    get post_path(post_id: posts(:draft).id,
                  blog_name: posts(:draft).blog.urlName)
    assert_select "title", posts(:draft).blog.title
  end

  test "return post if post is draft and user is logged in as author" do
    post login_path(login: {username: "user_one", password: "password1"})
    get post_path(post_id: posts(:draft).id,
                  blog_name: posts(:draft).blog.urlName)
    assert_response :success
  end

  test "draft post has draft in title" do
    post login_path(login: {username: "user_one", password: "password1"})
    get post_path(post_id: posts(:draft).id,
                  blog_name: posts(:draft).blog.urlName)
    assert_select "title", "[Draft] My draft post | The first blog"
  end
  
  test "show post if it exists and published" do
    get post_path(post_id: posts(:one).id,
                  blog_name: posts(:one).blog.urlName)
    assert_response :success
  end

  test "published post does not have draft in title" do
    get post_path(post_id: posts(:one).id,
                  blog_name: posts(:one).blog.urlName)
    assert_select "title", "My first sample post | The first blog"
  end
  
  test "can't visit new post page if not logged in" do
    get new_post_path(blog_name: blogs(:one).urlName)
    assert_response :forbidden
  end

  test "can't visit new post page if logged in" do
    post login_path(login: {username: "user_one", password: "password1"})
    get new_post_path(blog_name: blogs(:two).urlName)
    assert_response :forbidden
  end

  test "can visit new post page if logged in as correct user" do
    post login_path(login: {username: "user_two", password: "password2"})
    get new_post_path(blog_name: blogs(:two).urlName)
    assert_response :success
  end

  test "can't create new post if logged out" do
    pLast = Post.last.id
    post new_post_path(blog_name: blogs(:two).urlName,
                       post: {title: "My new post",
                              body: "My new post body"})
    assert_response :forbidden
    assert_equal pLast, Post.last.id
  end

  test "can't create new post if logged in as wrong user" do
    post login_path(login: {username: "user_one", password: "password1"})
    pLast = Post.last.id
    post new_post_path(blog_name: blogs(:two).urlName,
                       post: {title: "My new post",
                              body: "My new post body"})
    assert_response :forbidden
    assert_equal pLast, Post.last.id
  end

  test "can't create new post if no blog" do
    pLast = Post.last.id
    post login_path(login: {username: "user_two", password: "password2"})
    post new_post_path(blog_name: "does_not_exist",
                       post: {title: "My new post",
                              body: "My new post body"})
    assert_response :forbidden
    assert_equal pLast, Post.last.id
  end

  test "can create new post if logged in as blog author" do
    post login_path(login: {username: "user_one", password: "password1"})
    pLast = Post.last.id
    post new_post_path(blog_name: blogs(:one).urlName,
                       post: {title: "My new post",
                              body: "My new post body"})
    assert_redirected_to edit_post_path(post_id: Post.last.id)
    assert_not_equal pLast, Post.last.id
    assert_equal "My new post", Post.last.title
    assert_equal "My new post body", Post.last.body
  end

  test "can't visit edit post page if not logged in" do
    get edit_post_path(blog_name: blogs(:one).urlName,
                        post_id: posts(:one).id)
    assert_response :forbidden
  end

  test "can't visit edit post page if logged in as wrong user" do
    post login_path(login: {username: "user_two", password: "password2"})
    get edit_post_path(blog_name: blogs(:one).urlName,
                       post_id: posts(:one).id)
    assert_response :forbidden
  end

  test "can visit edit post page if logged in as author" do
    post login_path(login: {username: "user_one", password: "password1"})
    get edit_post_path(blog_name: blogs(:one).urlName,
                       post_id: posts(:one).id)
    assert_response :success
  end

  test "can't update post if not logged in" do
    oTitle = posts(:one).title
    post edit_post_path(blog_name: blogs(:one).urlName,
                        post_id: posts(:one).id,
                        post: {title: "My post's new title"})
    assert_response :forbidden
    assert_equal oTitle, Post.find(posts(:one).id).title
  end

  test "can't update post if logged in as wrong user" do
    oTitle = posts(:one).title
    post login_path(login: {username: "user_two", password: "password2"})
    post edit_post_path(blog_name: blogs(:one).urlName,
                        post_id: posts(:one).id,
                        post: {title: "My post's new title"})
    assert_response :forbidden
    assert_equal oTitle, Post.find(posts(:one).id).title
  end

  test "can update post if logged in as correct user" do
    post login_path(login: {username: "user_one", password: "password1"})
    post edit_post_path(blog_name: blogs(:one).urlName,
                        post_id: posts(:one).id,
                        post: {title: "My post's new title"})
    assert_redirected_to edit_post_path
    assert_equal "My post's new title", Post.find(posts(:one).id).title
  end

  test "can update post created_at date" do
    post login_path(login: {username: "user_one", password: "password1"})
    post edit_post_path(blog_name: blogs(:one).urlName,
                        post_id: posts(:one).id,
                        post: {created_at: "August 23, 1908"})
    assert_redirected_to edit_post_path
    assert_equal Date.parse("August 23, 1908"),
                 Post.find(posts(:one).id).created_at
  end
end
