require 'test_helper'

class UsersControllerTest < ActionDispatch::IntegrationTest
  test "new user page returns 200" do
    get signup_path
    assert_response :ok
  end

  test "create user creates user on success" do
    post users_path(
           user: {
             display_name: "Another user",
             username: "anotheruser",
             email: "abc@scd31.com",
             password: "test1234",
             password_confirmation: "test1234"
           }
         )
    u = User.find_by(username: "anotheruser")
    assert_equal "Another user", u.display_name
    assert_equal "abc@scd31.com", u.email
    assert u.authenticate("test1234")
  end
  
  test "create user redirects to root_path on success" do
    post users_path(
           user: {
             display_name: "Another user",
             username: "anotheruser",
             email: "abc@scd31.com",
             password: "test1234",
             password_confirmation: "test1234"
           }
         )
    assert_redirected_to root_path
  end

  test "create user flashes notice on success" do
    post users_path(
           user: {
             display_name: "Another user",
             username: "anotheruser",
             email: "abc@scd31.com",
             password: "test1234",
             password_confirmation: "test1234"
           }
         )
    assert_equal "Account created successfully!", flash[:notice]
  end

  test "create user automatically signs in on success" do
    post users_path(
           user: {
             display_name: "Another user",
             username: "anotheruser",
             email: "abc@scd31.com",
             password: "test1234",
             password_confirmation: "test1234"
           }
         )
    get dashboard_path
    assert_response :success
  end

  test "create user doesn't create user on failure" do
    post users_path(
           user: {
             display_name: "Another user",
             username: "anotheruser",
             email: "abc@scd31.com",
             password: "test1234",
             password_confirmation: "test123"
           }
         )
    assert_nil User.find_by(username: "anotheruser")
  end

  test "create user doesn't redirect on failure" do
    post users_path(
           user: {
             display_name: "Another user",
             username: "anotheruser",
             email: "abc@scd31.com",
             password: "test1234",
             password_confirmation: "test123"
           }
         )
    assert_response :ok
  end
  
  test "create user flashes alert on failure" do
    post users_path(
           user: {
             display_name: "Another user",
             username: "anotheruser",
             email: "abc@scd31.com",
             password: "test1234",
             password_confirmation: "test123"
           }
         )
    assert_equal ["Password confirmation doesn't match password"], flash[:alerts]
  end

  test "dashboard 403 if not logged in" do
    get dashboard_path
    assert_response :forbidden
  end

  test "dashboard 200 if logged in" do
    post login_path(login: {username: "user_one", password: "password1"})
    get dashboard_path
    assert_response :success
  end

  test "can't upload image if not logged in" do
    post upload_image_path
    assert_response :forbidden
  end

  test "can upload image if logged in" do
    post login_path(login: {username: "user_one", password: "password1"})
    post upload_image_path
    assert_redirected_to dashboard_path
  end

  test "can't upload video if not logged in" do
    post upload_video_path
    assert_response :forbidden
  end

  test "can upload video if logged in" do
    post login_path(login: {username: "user_one", password: "password1"})
    post upload_video_path
    assert_redirected_to dashboard_path
  end

  test "can't visit edit user page if not logged in" do
    get edit_user_path
    assert_response :forbidden
  end

  test "can visit edit user page if logged in" do
    post login_path(login: {username: "user_one", password: "password1"})
    get edit_user_path
    assert_response :success
  end

  test "can't update user if not logged in" do
    patch edit_user_path
    assert_response :forbidden
  end

  test "can update user if logged in" do
    post login_path(login: {username: "user_one", password: "password1"})
    patch edit_user_path(user: {display_name: "New name"})
    assert_response :success
    assert_equal "New name", User.find(users(:one).id).display_name
  end

  test "can't change password if not logged in" do
    patch change_password_path
    assert_response :forbidden
  end

  test "can't change password if current password is incorrect" do
    post login_path(login: {username: "user_one", password: "password1"})
    patch change_password_path(user: {old_password: "password2", password: "newpassword"})
    assert_response :ok
    u = User.find(users(:one).id)
    assert_not u.authenticate("newpassword")
    assert u.authenticate("password1")
  end

  test "can change password if logged in" do
    post login_path(login: {username: "user_one", password: "password1"})
    patch change_password_path(user: {old_password: "password1", password: "newpassword"})
    assert_response :success
    u = User.find(users(:one).id)
    assert_not u.authenticate("password1")
    assert u.authenticate("newpassword")
  end

  test "can't access admin dashboard if not logged in" do
    get admin_dashboard_path
    assert_response :forbidden
  end

  test "can't access admin dashboard if logged in as user" do
    post login_path(login: {username: "user_one", password: "password1"})
    get admin_dashboard_path
    assert_response :forbidden
  end

  test "can access admin dashboard if logged in as admin" do
    post login_path(login: {username: "user_admin", password: "admin_password"})
    get admin_dashboard_path
    assert_response :ok
  end

  test "user summary returns 404 when user does not exist" do
    get user_path(uname: "does-not-exist")
    assert_response :not_found
  end

  test "user summary returns 200 when user exists" do
    get user_path(uname: users(:one).username)
    assert_response :success
  end

  test "user summary display name by default" do
    get user_path(uname: users(:one).username)
    assert_equal assigns(:name), users(:one).display_name
  end

  test "user summary username if blank display name" do
    get user_path(uname: users(:no_displayname).username)
    assert_equal assigns(:name), users(:no_displayname).username
  end

  test "user summary does not include deleted comments" do
    get user_path(uname: users(:one).username)
    assert_not_includes assigns(:comments), comments(:deleted)
  end

  test "user summary does not include comment on draft" do
    get user_path(uname: users(:one).username)
    assert_not_includes assigns(:comments), comments(:draft)
  end
  
  test "user summary does not include draft posts" do
    get user_path(uname: users(:one).username)
    assert_not_includes assigns(:posts), posts(:draft)
  end
end
