require 'test_helper'

class UserTest < ActiveSupport::TestCase
  # viewable_posts
  
  test "viewable_posts does not return posts of other users" do
    assert_not_includes users(:one).viewable_posts(nil),
                        posts(:two)
  end
  
  test "viewable_posts returns published posts" do
    assert_includes users(:one).viewable_posts(nil),
                    posts(:one)
  end

  test "viewable_posts does not return draft posts when user is not nil" do
    assert_not_includes users(:one).viewable_posts(users(:two)),
                        posts(:draft)
  end

  test "viewable_posts does not return draft posts when user is nil" do
    assert_not_includes users(:one).viewable_posts(nil),
                        posts(:draft)
  end

  test "viewable_posts returns draft posts when user is author" do
    assert_includes users(:one).viewable_posts(users(:one)),
                    posts(:draft)
  end

  test "viewable_posts returns draft posts when user is admin" do
    assert_includes users(:one).viewable_posts(users(:admin)),
                    posts(:draft)
  end

  # viewable_comments

  test "viewable_comments does not return comments of other users" do
    assert_not_includes users(:one).viewable_comments(nil),
                        comments(:two)
  end
  
  test "viewable_comments returns comments on published posts" do
    assert_includes users(:one).viewable_comments(nil),
                    comments(:one)
  end

  test "viewable_comments does not return comments on draft posts when user is not nil" do
    assert_not_includes users(:one).viewable_comments(users(:three)),
                        comments(:draft)
  end

  test "viewable_comments does not return comments on draft posts when user is nil" do
    assert_not_includes users(:one).viewable_comments(nil),
                        comments(:draft)
  end

  test "viewable_comments does not return comments on draft post when " +
       "user is comment author but not post author" do
    assert_not_includes users(:one).viewable_comments(users(:one)),
                        comments(:draft)
  end
  
  test "viewable_comments returns comments on draft posts when user is author" do
    assert_includes users(:one).viewable_comments(users(:two)),
                    comments(:draft)
  end

  test "viewable_comments returns draft posts when user is admin" do
    assert_includes users(:one).viewable_comments(users(:admin)),
                    comments(:draft)
  end

  test "f_errors formats errors properly" do
    u = User.new
    u.save
    assert_equal ["Password can't be blank",
                  "Password is too short (minimum is 6 characters)",
                  "Email can't be blank",
                  "Email is invalid",
                  "Display name can't be blank",
                  "Username can't be blank"],
                 u.f_errors
      
  end

  test "f_errors returns empty array when no errors" do
    u = User.new
    assert_equal [], u.f_errors
  end

  test "color returns empty string when user is user" do
    u = users(:one)
    assert_equal "", u.color
  end

  test "color returns red when user is admin" do
    u = users(:admin)
    assert_equal "#FF3333", u.color
  end

  test "color returns empty string when user is unknown role" do
    u = users(:weird_role)
    assert_equal "", u.color
  end

  test "user can't create blog if allow_user_blogs is disabled and user is admin" do
    Rails.configuration.pressr.allow_user_blogs = false
    Rails.application.reload_routes!
    u = users(:one)
    assert_equal false, u.can_create_blog
  end

  test "user can create blog if allow_user_blogs is disabled and user is admin" do
    Rails.configuration.pressr.allow_user_blogs = false
    Rails.application.reload_routes!
    u = users(:admin)
    assert_equal true, u.can_create_blog
  end
  
  test "user can create blog if allow_user_blogs is enabled" do
    Rails.configuration.pressr.allow_user_blogs = true
    Rails.application.reload_routes!
    u = users(:one)
    assert_equal true, u.can_create_blog
  end

  test "admin? false when user is user" do
    assert_equal false, users(:one).admin?
  end

  test "admin? true when user is admin" do
    assert_equal true, users(:admin).admin?
  end
end
